<?php

namespace Drupal\facets_short_pretty_paths\Plugin\facets_pretty_paths\coder;

use Drupal\facets_pretty_paths\Coder\CoderPluginBase;
use Drupal\taxonomy\Entity\Term;

/**
 * Taxonomy machine name facets pretty paths coder.
 *
 * @FacetsPrettyPathsCoder(
 *   id = "taxonomy_term_machine_name_coder",
 *   label = @Translation("Taxonomy term machine name"),
 *   description = @Translation("Use term machine name (with a fallback to the term name with id if not available), e.g. /color/<strong>blue</strong>")
 * )
 */
class TaxonomyTermCoder extends CoderPluginBase {

  /**
   * Encode an id into an alias.
   *
   * @param string $id
   *   An entity id.
   *
   * @return string
   *   An alias.
   */
  public function encode($id) {
    $result = $id;
    if ($term = Term::load($id)) {
      if (!$term->get('machine_name')->isEmpty()) {
        $result = $term->get('machine_name')->value;
      }
      // Fallback to term name with id.
      else {
        $term_name = $term->get('name')->value;
        $term_name = \Drupal::service('pathauto.alias_cleaner')
          ->cleanString($term_name);
        $result = $term_name . '-' . $id;
      }
    }
    return $result;
  }

  /**
   * Decodes an alias back to an id.
   *
   * @param string $alias
   *   An alias.
   *
   * @return string
   *   An id.
   */
  public function decode($alias) {
    $result = '';
    // Fallback to id if machine name is not available.
    $exploded = explode('-', $alias);
    if (count($exploded) > 1) {
      $result = array_pop($exploded);
    }
    // If it does not contain a dash / id, the machine name is available.
    else {
      $termStorage = \Drupal::entityTypeManager()->getStorage('taxonomy_term');
      $query = $termStorage->getQuery()
        ->condition('machine_name', $alias);
      $termIds = $query->execute();
      if (!empty($termIds)) {
        $result = reset($termIds);
      }
    }
    return $result;
  }

}
